@@ -14,6 +14,7 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <unistd.h>
 #include <string.h>
@@ -30,12 +31,12 @@ extern struct libbase libdata;
    the prginitab. It does module data initialisation */
 
 int
-ipc_init (int mode, int reg)
+ipc_init (char *name, int mode, int flags)
 {
 	if (mode == LIBMODE_SERVER)
 		return ipc_serverinit ();
 	else
-		return ipc_clientinit (reg);
+		return ipc_clientinit (name, flags);
 }
 
 int
@@ -75,18 +76,25 @@ ipc_serverinit ()
 }
 
 int
-ipc_clientinit (int reg)
+ipc_clientinit (char *name, int flags)
 {
 	struct libbase *base = &libdata;
+	struct tagitem taglist[4];
 
 	base->mode = LIBMODE_CLIENT;
+	
+	taglist_init (taglist);
+	taglist_add (taglist, TAG_CLIENTFLAGS, (long) flags);
+	if (name)
+		taglist_add (taglist, TAG_CLIENTNAME, (long) name);
+	taglist_add (taglist, TAG_CLIENTPID, (long) getpid());
 
 	if ((base->serverport = ipc_findport(SERVERPORTKEY)) < 0) {
 		return E_NOSERVER;
 	} else if ((base->msgport = ipc_createport( (key_t) getpid() )) < 0) {
 		return E_MSGPORT;
-	} else if (reg == 1)
-		if ((ipc_send (0, REGISTERCLIENT, NULL)) < 0) {
+	} else if (flags & CLIENT_REGISTER)
+		if ((ipc_send (0, REGISTERCLIENT, taglist)) < 0) {
 			return E_REGISTER;
 	}
 	return 0;
@@ -187,16 +195,6 @@ ipc_receive (void *buffer, size_t bufferlen)
 			ipc_getmessagepart (base->msgport, 0, pbbmsg, bufferlen);
 	} else if (pbbmsg->messagetype == MESSAGETYPE) {
 		owner = ipc_getportowner (pbbmsg->returnport);
-		if (base->mode == LIBMODE_SERVER) {
-			if (pbbmsg->action == REGISTERCLIENT) {
-				if (register_client (pbbmsg->returnport))
-					ipc_send (pbbmsg->returnport, REGFAILED, NULL);
-				return -1;  /* message evaluated */
-			} else if (pbbmsg->action ==  UNREGISTERCLIENT) {
-				unregister_client (pbbmsg->returnport);
-				return -1;  /* message evaluated */
-			}
-		}
 		n = 0;
 		while (pbbmsg->taglist[n].tag) {
 			if ((pbbmsg->action == CHANGEVALUE) && (base->filtermode == 1))
@@ -205,7 +203,7 @@ ipc_receive (void *buffer, size_t bufferlen)
 				tagerror (&pbbmsg->taglist[n], E_PERM);
 			else if ((pbbmsg->action == CHANGEVALUE) && (base->filtermode == 0) &&
 				  (tagfind (base->ptags, pbbmsg->taglist[n].tag, 0)) &&
-				  (owner != geteuid()))                                /* protected tags are only changable by*/
+				  (owner != geteuid()))                /* protected tags are only changable by */
 				tagerror (&pbbmsg->taglist[n], E_PERM);   /* the processowner of pbbuttonsd */
 			else if ((pbbmsg->taglist[n].tag & FLG_PRIVATE))     /* filter private tags */
 				tagerror (&pbbmsg->taglist[n], E_PRIVATE);
@@ -214,45 +212,162 @@ ipc_receive (void *buffer, size_t bufferlen)
 				pbbmsg->taglist[n].data += (long) buffer;
 			n++;
 		}
+		if (base->mode == LIBMODE_SERVER) {
+			if (pbbmsg->action == REGISTERCLIENT) {
+				if (register_client (pbbmsg->returnport, pbbmsg->taglist))
+					ipc_send (pbbmsg->returnport, REGFAILED, NULL);
+				return -1;  /* message evaluated */
+			} else if (pbbmsg->action ==  UNREGISTERCLIENT) {
+				unregister_client (pbbmsg->returnport);
+				return -1;  /* message evaluated */
+			} else if (pbbmsg->action == GETCLIENTLIST) {
+				ipc_sendclientlist(pbbmsg->returnport);
+				return -1;  /* message evaluated */
+			}
+		}	
 		return 0;   /* message received */
 	}
 	return -1;  /* no message pending or ignore it */
 }
 
 /* This function adds a client message port to the list. This client will get event messages
-   from the server in the future. If the client was successfull added, the return value
-   would be 0, otherwise -1 */
-
-int
-register_client (int cmsgport)
+ * from the server in the future. If the client was successfull added, the return value
+ * would be 0, otherwise -1
+ */
+#ifdef DEBUG
+void
+printClients ()
 {
 	struct libbase *base = &libdata;
+	struct listnode *it = base->clients.first;
+	struct pbbclient *client;
 
-	if (base->count < (MAXCLIENTS -1)) {
-		base->client[(base->count)++] = cmsgport;
+	printf ("\nClients:\n\n");
+	printf ("  PID   | Name                 | Port    | Flags\n");
+	printf ("  ------+----------------------+---------+-----------\n");
+	while ((it = it->next)) {
+		client = (struct pbbclient *) getNodeData (it->pred);
+		printf ("  %5d | %-20s | %7d | %08x\n", client->pid,
+				(client->name) ? client->name : "<unknown>",
+				client->port, client->flags);
+	}
+	printf ("\n");
+}
+#endif
+
+void
+ipc_sendclientlist (int port)
+{
+	struct libbase *base = &libdata;
+	struct listnode *it = base->clients.first;
+	struct pbbclient *client;
+	struct tagitem *taglist;
+	int cnt;
+
+	cnt = getNodeCount(&base->clients);
+	if ((taglist = (struct tagitem *) malloc (sizeof(struct tagitem) * (cnt*4 + 1)))) {
+		taglist_init (taglist);
+	
+		while ((it = it->next)) {
+			client = (struct pbbclient *) getNodeData (it->pred);
+			taglist_add (taglist, TAG_CLIENTPID,   client->pid);
+			taglist_add (taglist, TAG_CLIENTNAME,  (tag_t) client->name);
+			taglist_add (taglist, TAG_CLIENTPORT,  client->port);
+			taglist_add (taglist, TAG_CLIENTFLAGS, client->flags);
+		}
+		
+		ipc_send (port, CHANGEVALUE, taglist);
+		free (taglist);
+	}
+}
+
+struct listnode *
+getPBBClientName (struct listhead *lh, char *name)
+{
+	struct listnode *it = lh->first;
+	struct pbbclient *client;
+
+	if (name) {
+		while ((it = it->next)) {
+			client = (struct pbbclient *) getNodeData (it->pred);
+			if (client->name)
+				if (strcmp (name, client->name) == 0)
+					return it->pred;
+		}		
+	}
+	return NULL;
+}
+
+int
+register_client (int cmsgport, struct tagitem *taglist)
+{
+	struct libbase *base = &libdata;
+	struct listnode *oldnode, *newnode;
+	struct pbbclient *client;
+	long flags;
+	char *name;
+	int len, pid;
+
+	name = (char *) tagfind (taglist, TAG_CLIENTNAME, (long) NULL);
+	flags = tagfind (taglist, TAG_CLIENTFLAGS, 0);
+	pid = (int) tagfind (taglist, TAG_CLIENTPID, 0);
+
+	oldnode = getPBBClientName (&base->clients, name);
+	if ((oldnode) && (flags & CLIENT_EXCLUSIVE)) { 
+		if (flags & CLIENT_REPLACEMENT) {
+			/* new client replaces old one. So wave old existing
+			 * client good bye
+			 */
+			client = (struct pbbclient *) getNodeData (oldnode);
+			ipc_send (client->port, CLIENTEXIT, NULL);
+			freeListNode (oldnode);
+		} else
+			/* client already exists, registration denied */
+			return -1;
+	}
+	
+	len = (name) ? strlen (name) + 1 : 0;
+	if ((newnode = newListNode (sizeof (struct pbbclient) + len))) {
+		client = (struct pbbclient *) getNodeData (newnode);
+		client->port = cmsgport;
+		client->flags = flags;
+		client->pid = pid;
+		if (name) {
+			client->name = (char*) client + sizeof (struct pbbclient);
+			strcpy (client->name, name);
+		} else
+			client->name = NULL;
+		appendListNode (&base->clients, newnode);
+#ifdef DEBUG
+		printClients();
+#endif
 		return 0;
-	} else
-		return -1;
+	}
+	return -1;
 }
 
 /* This function removes a client message port from the list and reorganizes the
-   list so that gaps would be eliminated. This client won't get further messages from
-   the server. If the client was successfull removed, the return value would be 0,
-   otherwise -1 */
-
+ * list so that gaps would be eliminated. This client won't get further messages from
+ * the server. If the client was successfull removed, the return value would be 0,
+ * otherwise -1
+ */
 int
 unregister_client (int cmsgport)
 {
 	struct libbase *base = &libdata;
-	int n;
+	struct listnode *it = base->clients.first;
+	struct pbbclient *client;
 
-	for (n=0; n < base->count; n++)
-		if (base->client[n] == cmsgport) {
-			for ((base->count)--; n < base->count; n++)
-				base->client[n] = base->client[n+1];
-			base->client[base->count] = 0;
+	while ((it = it->next)) {
+		client = (struct pbbclient *) getNodeData (it->pred);
+		if (client->port == cmsgport) {
+			freeListNode (it->pred);
+#ifdef DEBUG	
+			printClients();
+#endif
 			return 0;
 		}
+	}
 	return -1;
 }
 
@@ -282,21 +397,23 @@ singletag_to_server (int action, tag_t tag, tag_t data)
 }
 
 /* This function sends a message with given code and value to every registered
-    client. If an error with a client occured, that client would be unregistered */
-
+ * client. If an error with a client occured, that client would be unregistered
+ */
 void
 distribute_to_clients (int action, struct tagitem *taglist)
 {
 	struct libbase *base = &libdata;
-	int n;
+	struct listnode *it = base->clients.first;
+	struct pbbclient *client;
 
 #ifdef DEBUG
 	peep_ipc (taglist);
 #endif
 
-	for (n=0; n < base->count; n++) {
-		if ((ipc_send (base->client[n], action, taglist)) != 0)
-			unregister_client (base->client[n--]);
+	while ((it = it->next)) {
+		client = (struct pbbclient *) getNodeData (it->pred);
+		if ((ipc_send (client->port, action, taglist)) != 0)
+			unregister_client (client->port);
 	}
 }
 
