@@ -188,7 +188,7 @@ set_timeforcmd(int timeout)
 }
 
 int
-check_permissions (char *file, int uid, int perms, int mask)
+check_permissions (char *file, uid_t uid, mode_t perms, int mask)
 {
 	struct stat stat_buf;
 
@@ -271,7 +271,8 @@ cleanup_buffer(char *buffer)
 int
 axtoi (char *arg)
 {
-	int n, val, pwr=1, m, rc = 0;
+	int val, pwr = 1, rc = 0;
+	size_t n, m;
 	char hex[9], c;
 
 	for (n=0,m=0; n < strlen(arg); n++)
