@@ -41,6 +41,8 @@
 #  include "module_pmac.h"
 #endif
 
+#include "class_backlight.h"
+
 #include "module_system.h"
 #include "module_powersave.h"
 #include "module_display.h"
@@ -70,6 +72,8 @@ init_serverdata (struct serverdata *sd)
 	sd->module[n].openfunc = ipc_init_stub;      sd->module[n++].closefunc = ipc_exit;
 	sd->module[n].initfunc = system_init;        sd->module[n].exitfunc    = system_exit;        /* pri 1 */
 	sd->module[n].openfunc = system_open;        sd->module[n++].closefunc = system_close;
+	sd->module[n].initfunc = backlight_init;     sd->module[n].exitfunc    = backlight_exit;     /* pri 1 */
+	sd->module[n].openfunc = backlight_open;     sd->module[n++].closefunc = backlight_close;
 #ifdef WITH_MODULE_PMAC
 	sd->module[n].initfunc = pmac_init;          sd->module[n].exitfunc    = pmac_exit;          /* pri 1 */
 	sd->module[n].openfunc = pmac_open;          sd->module[n++].closefunc = pmac_close;
@@ -369,6 +373,6 @@ ipc_init_stub (struct tagitem *taglist)
 		break;
 	}
 
-	register_function (T10QUEUE, ipc_handler);  /* input via IPC messages */
+	register_function (T100QUEUE, ipc_handler);  /* input via IPC messages */
 	return err;
 }
