@@ -12,6 +12,8 @@
  * 2 of the License, or (at your option) any later version.
  * ----------------------------------------------------------------------------*/
 
+#include <glib.h>
+
 /* short options for optarg */
 #define ARG_ALL			"hvqd::c:"
 #define ARG_HELP		'h'
@@ -30,6 +32,7 @@ struct serverdata {
 	                         * Writing the configfile will be controlled by
 	                         * module_system.
 	                         */
+	GMainLoop *mainloop;
 	int quiet;
 	struct {
 		int (*initfunc)();
@@ -45,8 +48,9 @@ int init_modules(struct serverdata *sd);
 void exit_modules(struct serverdata *sd);
 void reconfig_modules(struct serverdata *sd);
 int evaluate_args(struct serverdata *sd, int argc, char *argv[]);
-void sighandler_cleanup (int signum);
-int install_sighandler ();
+void cbSigHandlerTX (int signum);
+gboolean cbSigHandlerRD (int fd, gpointer user_data);
+void installSigHandler (struct serverdata *base);
 int ipc_init_stub ();
 
 #endif    /* INCLUDE_INIT_H */
