@@ -39,10 +39,10 @@ struct moddata_alsamixer {
 	unsigned short modvoldn;
 	unsigned short keymute;
 	unsigned short modmute;
-	int init_complete;
-	int mixer_delayed;
-	int mute;
-	int open;
+	unsigned int init_complete;
+	unsigned int mixer_delayed;
+	unsigned int mute;
+	unsigned int open;
 	snd_mixer_t *mixer;
 	snd_mixer_elem_t **elements; /* points to a NULL terminated list, first is master */
 } modbase_alsamixer;
@@ -168,7 +168,7 @@ alsamixer_setup_elements_internal (char * channels, snd_mixer_selem_id_t *sid)
 
 		/* check that master element has a playback volume */
 		if (!snd_mixer_selem_has_playback_volume (base->elements[0])) {
-			snd_mixer_selem_get_id (base->elements[n], sid);
+			snd_mixer_selem_get_id (base->elements[0], sid);
 			print_msg (PBB_WARN, _("Mixer element '%s' has no playback volume control.\n"), snd_mixer_selem_id_get_name (sid));
 			return -1;
 		}
