@@ -15,9 +15,12 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <string.h>
+#include <strings.h>
+#include <time.h>
+#include <math.h>
 #include "pbbinput.h"
 #include <pbb.h>
 
@@ -29,6 +32,12 @@
 
 #include <linux/radeonfb.h>
 
+#include "debug.h"
+
+#define VAL2PERCENT(x,max)  (x * 100 / max)
+#define PERCENT2VAL(x,max)  (x * max / 100)
+
+
 struct moddata_display {
 	char *fbdev;         /* pathname of the framebuffer device */
 	struct display_flags flags;
@@ -72,7 +81,7 @@ display_init ()
 		return E_NOMEM;
 	} else {
 		registerCfgOptionInt (sid, "LCD_Brightness", TAG_LCDBRIGHTNESS, 0,
-				N_("initial LCD brightness level"));
+				N_("initial LCD brightness level in percent"));
 		registerCfgOptionInt (sid, "LCD_FadingSpeed", TAG_LCDFADINGSPEED, 0,
 				N_("0 = no smooth fading"));
 		registerCfgOptionBool (sid, "LCD_AutoAdjust", TAG_LCDAUTOADJUST, 0,
@@ -92,9 +101,9 @@ display_init ()
 		registerCfgOptionInt (sid, "LCD_AutoAdjMax_AC", TAG_LCDAUTOADJMAXAC, 0,
 				NULL);
 		registerCfgOptionInt (sid, "KBD_Brightness", TAG_KBDBRIGHTNESS, 0,
-				N_("initial keyboard illumination level"));
+				N_("initial keyboard illumination level in percent"));
 		registerCfgOptionInt (sid, "KBD_OnBrightness", TAG_KBDONBRIGHTNESS, 0,
-				N_("initial level if KBD on/off key is pressed"));
+				N_("initial level in percent if KBD on/off key is pressed"));
 		registerCfgOptionInt (sid, "KBD_FadingSpeed", TAG_KBDFADINGSPEED, 0,
 				N_("0 = no smooth fading"));
 		registerCfgOptionBool (sid, "KBD_AutoAdjust", TAG_KBDAUTOADJUST, 0,
@@ -150,18 +159,18 @@ display_open (struct tagitem *taglist)
 	base->lcdlight.max         = (int) tagfind (args, TAG_BACKLIGHTMAX, 0);
 	base->lcdlight.backup      = level;
 	base->lcdlight.offset      = 0;
-	base->lcdlight.fadingspeed = 0;
-	base->lcdlight.fadingcount = 0;
 	base->lcdlight.threshold   = 94; /* value in percent */
+	base->lcdlight.isrunning   = 0;
+	display_set_fading (&base->lcdlight, (int) tagfind (taglist, TAG_LCDFADINGSPEED, 0));
 
 	base->kbdlight.current     = 0;
 	base->kbdlight.target      = 0;
 	base->kbdlight.max         = (int) tagfind (args, TAG_KEYBLIGHTMAX, 0);
 	base->kbdlight.backup      = 0;
 	base->kbdlight.offset      = 0;
-	base->kbdlight.fadingspeed = 5;
-	base->kbdlight.fadingcount = 0;
 	base->kbdlight.threshold   = 28;  /* value in percent */
+	base->kbdlight.isrunning   = 0;
+	display_set_fading (&base->kbdlight, (int) tagfind (taglist, TAG_KBDFADINGSPEED, 0));
 	process_queue_single (CONFIGQUEUE, TAG_KEYBLIGHTLEVEL, base->kbdlight.current);
 
 	base->keylcdup	= KEY_BRIGHTNESSUP;
@@ -182,10 +191,10 @@ display_open (struct tagitem *taglist)
 	base->autoadj_min_ac  = 1;
 	base->autoadj_max_ac  = base->lcdlight.max;
 	
-	base->kbdon_brightness = KBD_BRIGHTNESS_MIN; /* saved brighhtness for kbd on/off key */
+	base->kbdon_brightness = KBD_BRIGHTNESS_MIN; /* saved brightness for kbd on/off key */
 	
 	base->flags.status 	   = STATUS_NORMAL;
-	base->flags.lmu_enable	   = tagfind(args, TAG_AMBIENTLIGHT, -1) == -1 ? 0 : 1;
+	base->flags.lmu_enable	   = tagfind(args, TAG_AMBIENTLIGHT, 0xffffffff) == 0xffffffff ? 0 : 1;
 	base->flags.ctrl_fb        = 0;  /* framebuffer disabled by default  */
 	base->flags.dimfullydark   = 0;  /* don't dim to complete darkness by default */
 	base->flags.lcd_autoadjust = 1;  /* lcd and keyboard should react on */
@@ -204,7 +213,6 @@ display_open (struct tagitem *taglist)
 		base->flags.kbd_switchedoff= !base->flags.kbd_autoadjust; 
 
 	register_function (KBDQUEUE, display_keyboard);
-	register_function (T10QUEUE, display_timer);
 	register_function (T1000QUEUE, display_ambienttimer);
 	return 0;
 }
@@ -241,13 +249,17 @@ void
 display_keyboard (struct tagitem *taglist)
 {
 	struct moddata_display *base = &modbase_display;
-	int code, value, mod, level = 0, changed = 0, tmp;
+	int code, value, mod, lcdstep, kbdstep, tmp;
+	int level = 0, changed = 0;
 
 	code = (int) tagfind (taglist, TAG_KEYCODE, 0);
 	value = (int) tagfind (taglist, TAG_KEYREPEAT, 0);
 	mod = (int) tagfind (taglist, TAG_MODIFIER, 0);
 
 	if (value && base->flags.coveropen) {
+		lcdstep = (base->lcdlight.max >> 4) + 1;
+		kbdstep = (base->kbdlight.max >> 4) + 1;
+
 		/* key for video mirroring */
 		if ((code == base->keymirror) && (mod == base->modmirror))
 			if ((tmp = display_switchmirror ()) != -1)
@@ -267,11 +279,11 @@ display_keyboard (struct tagitem *taglist)
 			 * yet (only .target has been set accordingly) and if we used
 			 * .current here we would overwrite it.
 			 */
-			level = mod & MOD_SHIFT ? base->lcdlight.max : base->lcdlight.target + 1;
+			level = mod & MOD_SHIFT ? base->lcdlight.max : base->lcdlight.target + lcdstep;
 			changed = 1;
 		} else if ((code == base->keylcddn) && ((mod & ~MOD_SHIFT) == base->modlcddn)) {
 			display_sync_lcdbrightness ();  /* same as brightness up */
-			level = mod & MOD_SHIFT ? LCD_BRIGHTNESS_MIN : base->lcdlight.target - 1;
+			level = mod & MOD_SHIFT ? LCD_BRIGHTNESS_MIN : base->lcdlight.target - lcdstep;
 			changed = 1;
 		}
 		
@@ -281,14 +293,14 @@ display_keyboard (struct tagitem *taglist)
 				base->kbdlight.current = base->kbdon_brightness;
 				base->flags.kbd_switchedoff = 0;
 			}
-			level = mod & MOD_SHIFT ? base->kbdlight.max : base->kbdlight.current + 1;
+			level = mod & MOD_SHIFT ? base->kbdlight.max : base->kbdlight.current + kbdstep;
 			changed = 2;
 		} else if ((code == base->keykbddn) && ((mod & ~MOD_SHIFT) == base->modkbddn)) {
 			if (base->flags.kbd_switchedoff) {
 				base->kbdlight.current = base->kbdon_brightness;
 				base->flags.kbd_switchedoff = 0;
 			}
-			level = mod & MOD_SHIFT ? KBD_BRIGHTNESS_MIN : base->kbdlight.current - 1;
+			level = mod & MOD_SHIFT ? KBD_BRIGHTNESS_MIN : base->kbdlight.current - kbdstep;
 			changed = 2;
 		} else if (base->flags.lmu_enable && (code == base->keykbdon) && (mod == base->modkbdon)) {
 			if (base->flags.kbd_switchedoff) {
@@ -300,7 +312,8 @@ display_keyboard (struct tagitem *taglist)
 				level = KBD_BRIGHTNESS_OFF;
 			}
 			level = display_set_kbdbrightness(level);
-			singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
+			singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS,
+					             VAL2PERCENT(level, base->kbdlight.max));
 		}
 
 		switch (changed) {
@@ -309,7 +322,7 @@ display_keyboard (struct tagitem *taglist)
 				if (mod & MOD_SHIFT)
 					break;   /* shift modifier disabled in automatic mode */
 				tmp = base->lcdlight.current - base->lcdlight.offset;
-				base->lcdlight.offset += (level - base->lcdlight.current) > 0 ? 1 : -1;
+				base->lcdlight.offset += (level - base->lcdlight.current) > 0 ? lcdstep : -lcdstep;
 				if (tmp + base->lcdlight.offset > base->lcdlight.max)
 					base->lcdlight.offset = base->lcdlight.max - tmp;
 				if (tmp + base->lcdlight.offset < 1)
@@ -318,7 +331,8 @@ display_keyboard (struct tagitem *taglist)
 				level = tmp + base->lcdlight.offset;
 			}
 			level = display_set_lcdbrightness(level);
-			singletag_to_clients(CHANGEVALUE, TAG_LCDBRIGHTNESS, level);
+			singletag_to_clients(CHANGEVALUE, TAG_LCDBRIGHTNESS,
+					             VAL2PERCENT(level, base->lcdlight.max));
 			break;
 		case 2:  /* Keyboard illumination level changed by keys */
 			if (base->flags.lmu_enable) {
@@ -326,7 +340,7 @@ display_keyboard (struct tagitem *taglist)
 					if (mod & MOD_SHIFT)
 						break;   /* shift modifier disabled in automatic mode */
 					tmp = base->kbdlight.current - base->kbdlight.offset;
-					base->kbdlight.offset += (level - base->kbdlight.current) > 0 ? 1 : -1;
+					base->kbdlight.offset += (level - base->kbdlight.current) > 0 ? kbdstep : -kbdstep;
 					if (tmp + base->kbdlight.offset > base->kbdlight.max)
 						base->kbdlight.offset = base->kbdlight.max - tmp;
 					if (tmp + base->kbdlight.offset < 0)
@@ -334,59 +348,95 @@ display_keyboard (struct tagitem *taglist)
 					level = tmp + base->kbdlight.offset;
 				}
 				level = display_set_kbdbrightness(level);
-				singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
+				singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS,
+						             VAL2PERCENT(level, base->kbdlight.max));
 			}
 			break;
 		}
 	}
 }
 
-/* Timer function called every 10ms.
- * The current brightness level for each lightsource will
- * be modified to match the appropriate target level. This
- * modification could be done hardly or smoothly depending
- * on certain parameters set or not.
- * This function will react automatically as soon as the
- * current brightness level differs from the target level.
+/* The following two timer functions will set the brightness
+ * level for the LCD and keyboard backlight. The timer will
+ * be dynamically configured to be able to switch the brightness
+ * hard or smooth from one level to another. 
+ * After the timer function did their job it will disable itself.
+ * A new brightness change request will launch another timer.
  */
-void
-display_timer (struct tagitem *taglist)
+gboolean
+display_lcdtimer (gpointer data)	
 {
 	struct moddata_display *base = &modbase_display;
+	struct display_light *illu = &base->lcdlight;
 
-	if (base->lcdlight.current != base->lcdlight.target) {
-		base->lcdlight.fadingcount++;
-		if (base->lcdlight.fadingcount >= base->lcdlight.fadingspeed) {
-			base->lcdlight.fadingcount = 0;
-			if (base->lcdlight.fadingspeed) {
-				if (base->lcdlight.current > base->lcdlight.target)
-					base->lcdlight.current--;   /* target is darker as current */
-				else
-					base->lcdlight.current++;   /* target is brighter as current */
-			} else
-				base->lcdlight.current = base->lcdlight.target;
-			process_queue_single (CONFIGQUEUE, TAG_BACKLIGHTLEVEL, base->lcdlight.current);
-			display_framebuffer (base->lcdlight.current ? 1: 0);
-		}
+	if (illu->current != illu->target) {
+		if (illu->fading && abs(illu->current - illu->target) >= illu->fadingstep) {
+			if (illu->current > illu->target)  /* target is darker as current */
+				illu->current -= illu->fadingstep; 
+			else                               /* target is brighter as current */
+				illu->current += illu->fadingstep;
+		} else
+			illu->current = illu->target;
+
+		process_queue_single (CONFIGQUEUE, TAG_BACKLIGHTLEVEL, illu->current);
+		display_framebuffer (illu->current ? 1: 0);
+		return TRUE;
 	}
 
-	if (base->flags.lmu_enable)
-		if (base->kbdlight.current != base->kbdlight.target) {
-			base->kbdlight.fadingcount++;
-			if (base->kbdlight.fadingcount >= base->kbdlight.fadingspeed) {
-				base->kbdlight.fadingcount = 0;
-				if (base->kbdlight.fadingspeed) {
-					if (base->kbdlight.current > base->kbdlight.target)
-						base->kbdlight.current--;   /* target is darker as current */
-					else
-						base->kbdlight.current++;   /* target is brighter as current */
-				} else
-					base->kbdlight.current = base->kbdlight.target;
-				process_queue_single (CONFIGQUEUE, TAG_KEYBLIGHTLEVEL, base->kbdlight.current);
-			}
-		}
+#if defined(DEBUG) && DISPLAYTIMER
+	struct timeval endtime;
+	double time;
+
+	gettimeofday (&endtime, NULL);
+	time = (endtime.tv_sec  - illu->start.tv_sec)  * 1000 +
+		   (endtime.tv_usec - illu->start.tv_usec) / 1000;
+
+	print_msg (PBB_INFO, "DBG: LCD Display total time: %fms, time per step: %fms\n",
+			        time, time / illu->steps); 
+	print_msg (PBB_INFO, "DBG: LCD Display Timer stopped.\n");
+#endif
+	illu->isrunning = 0;
+	return FALSE;
 }
 
+gboolean
+display_kbdtimer (gpointer data)	
+{
+	struct moddata_display *base = &modbase_display;
+	struct display_light *illu = &base->kbdlight;
+
+	if (base->flags.lmu_enable) {
+		if (illu->current != illu->target) {
+			if (illu->fading && abs(illu->current - illu->target) >= illu->fadingstep) {
+				if (illu->current > illu->target)  /* target is darker as current */
+					illu->current -= illu->fadingstep; 
+				else                               /* target is brighter as current */
+					illu->current += illu->fadingstep;
+			} else
+				illu->current = illu->target;
+
+			process_queue_single (CONFIGQUEUE, TAG_KEYBLIGHTLEVEL, illu->current);
+			return TRUE;
+		}
+
+#if defined(DEBUG) && DISPLAYTIMER
+		struct timeval endtime;
+		double time;
+
+		gettimeofday (&endtime, NULL);
+		time = (endtime.tv_sec  - illu->start.tv_sec)  * 1000 +
+			   (endtime.tv_usec - illu->start.tv_usec) / 1000;
+
+		print_msg (PBB_INFO, "DBG: KBD Display total time: %fms, time per step: %fms\n",
+				        time, time / illu->steps); 
+		print_msg (PBB_INFO, "DBG: KBD Display Timer stopped.\n");
+#endif
+	}
+	illu->isrunning = 0;
+	return FALSE;
+}
+
+
 /* Timer function called every second
  * In this function the automatic brighness adjustments for
  * LCD and keyboard are done.
@@ -465,45 +515,54 @@ display_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 		case TAG_LCDBRIGHTNESS:
 			if (cfgure) {
-				level = display_set_lcdbrightness (taglist->data);
-				singletag_to_clients(CHANGEVALUE, TAG_LCDBRIGHTNESS, level);
+				level = taglist->data;  /* level in percent */
+				if (level > 100) level = 100;
+				if (level < 0)   level = 0;
+				level = display_set_lcdbrightness (PERCENT2VAL(level, base->lcdlight.max));
+				singletag_to_clients(CHANGEVALUE, TAG_LCDBRIGHTNESS,
+						             VAL2PERCENT(level, base->lcdlight.max));
 			} else
-				taglist->data = base->flags.nobacklight ? -1 : base->lcdlight.current;
-			break;
-		case TAG_LCDBRIGHTNESSMAX:
-			if (cfgure)	tagerror (taglist, E_NOWRITE);
-			else		taglist->data = base->lcdlight.max;
+				taglist->data = base->flags.nobacklight ? -1 :
+					VAL2PERCENT(base->lcdlight.current, base->lcdlight.max);
 			break;
+//		case TAG_LCDBRIGHTNESSMAX:  /* deprecated */
+//			if (cfgure)	tagerror (taglist, E_NOWRITE);
+//			else		taglist->data = base->lcdlight.max;
+//			break;
 		case TAG_LCDFADINGSPEED:
-			if (cfgure)	base->lcdlight.fadingspeed = taglist->data;
-			else		taglist->data = base->lcdlight.fadingspeed;
+			if (cfgure)	display_set_fading (&base->lcdlight, taglist->data);
+			else		taglist->data = display_get_fading (&base->lcdlight);
 			break;
 		case TAG_KBDBRIGHTNESS:
 			if (cfgure) {
-				if ((level = display_set_kbdbrightness (taglist->data)) > 0)
-					base->flags.kbd_switchedoff = 0;
-				singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS, level);
+				level = taglist->data;  /* level in percent */
+				if (level > 100) level = 100;
+				if (level < 0)   level = 0;
+				level = display_set_kbdbrightness (PERCENT2VAL(level, base->kbdlight.max));
+				if (level > 0)   base->flags.kbd_switchedoff = 0;
+				singletag_to_clients(CHANGEVALUE, TAG_KBDBRIGHTNESS,
+						             VAL2PERCENT(level, base->kbdlight.max));
 			} else
-				taglist->data = base->kbdlight.current;
+				taglist->data = VAL2PERCENT(base->kbdlight.current, base->kbdlight.max);
 			break;
 		case TAG_KBDONBRIGHTNESS:
 			if (cfgure) {
-				level = taglist->data;
-				if (level < KBD_BRIGHTNESS_MIN)
-					level = KBD_BRIGHTNESS_MIN;
-				else if (level > base->kbdlight.max)
-					level = base->kbdlight.max;
-				base->kbdon_brightness = level;
+				level = taglist->data;  /* level in percent */
+				if (level > 100) level = 100;
+				if (level < 0)   level = 0;
+				base->kbdon_brightness = PERCENT2VAL(level, base->kbdlight.max);
+				if (base->kbdon_brightness < KBD_BRIGHTNESS_MIN)
+					base->kbdon_brightness = KBD_BRIGHTNESS_MIN;
 			} else
-				taglist->data = base->kbdon_brightness;
-			break;
-		case TAG_KBDBRIGHTNESSMAX:
-			if (cfgure)	tagerror (taglist, E_NOWRITE);
-			else		taglist->data = base->kbdlight.max;
+				taglist->data = VAL2PERCENT(base->kbdon_brightness, base->kbdlight.max);
 			break;
+//		case TAG_KBDBRIGHTNESSMAX:  /* deprecated */
+//			if (cfgure)	tagerror (taglist, E_NOWRITE);
+//			else		taglist->data = base->kbdlight.max;
+//			break;
 		case TAG_KBDFADINGSPEED:
-			if (cfgure)	base->kbdlight.fadingspeed = taglist->data;
-			else		taglist->data = base->kbdlight.fadingspeed;
+			if (cfgure)	display_set_fading (&base->kbdlight, taglist->data);
+			else		taglist->data = display_get_fading (&base->kbdlight);
 			break;
 		case TAG_LCDILLUMUPKEY:
 			if (cfgure)	base->keylcdup = taglist->data;
@@ -651,20 +710,54 @@ int
 display_set_lcdbrightness(int level)
 {
 	struct moddata_display *base = &modbase_display;
+	struct display_light *illu = &base->lcdlight;
+	int val;
 	
 	if (base->flags.nobacklight) {
-		if (level > 0) level = base->lcdlight.max;
-		base->lcdlight.target = level;
+		if (level > 0) level = illu->max;
+		illu->target = level;
 		return -1;
-	} else
-		return display_clip_brightness (&base->lcdlight, level);
+	} else {
+		val = display_clip_brightness (illu, level);
+		if (illu->isrunning == 0) {
+			illu->isrunning = 1;
+
+#if defined(DEBUG) && DISPLAYTIMER
+			illu->steps = abs(illu->target - illu->current);
+			gettimeofday (&illu->start, NULL);
+			print_msg (PBB_INFO, "DBG: LCD Display Timer started.\n");
+#endif
+			g_timeout_add (illu->fading ? illu->fading : 9, display_lcdtimer, NULL);
+		}
+#if defined(DEBUG) && BRIGHTNESS
+		print_msg (PBB_INFO, "DBG: LCD Brightness: %d\n", val);
+#endif
+		return val;
+	}
 }
 
 int
 display_set_kbdbrightness(int level)
 {
 	struct moddata_display *base = &modbase_display;
-	return display_clip_brightness (&base->kbdlight, level);
+	struct display_light *illu = &base->kbdlight;
+	int val;
+	
+	val = display_clip_brightness (illu, level);
+	if (illu->isrunning == 0) {
+		illu->isrunning = 1;
+
+#if defined(DEBUG) && DISPLAYTIMER
+		illu->steps = abs(illu->target - illu->current);
+		gettimeofday (&illu->start, NULL);
+		print_msg (PBB_INFO, "DBG: KBD Display Timer started.\n");
+#endif
+		g_timeout_add (illu->fading ? illu->fading : 9, display_kbdtimer, NULL);
+	}	
+#if defined(DEBUG) && BRIGHTNESS
+	print_msg (PBB_INFO, "DBG: KBD Brightness: %d\n", val);
+#endif
+	return val;
 }
 
 int
@@ -678,6 +771,35 @@ display_clip_brightness(struct display_light *illu, int level)
 	return level;
 }
 
+void
+display_set_fading (struct display_light *illu, int fading)
+{
+	double timer, step;
+
+	if (fading > 0) {
+		timer = (double) fading / (illu->max + 1);
+		step  = ceil (5.0 / timer);
+		if (step < illu->max) {
+			illu->fading = floor (timer * step + 0.5) - 1;
+			illu->fadingstep = step;
+			goto out;
+		}
+	}
+	illu->fading = 0;
+	illu->fadingstep = 1;
+out:
+#if defined(DEBUG) && DISPLAYTIMER
+	print_msg (PBB_INFO, "DBG: Timer: %d, step: %d\n", illu->fading, illu->fadingstep);
+#endif
+	return;
+}
+
+int
+display_get_fading (struct display_light *illu)
+{
+	return illu->fading * (illu->max + 1) / illu->fadingstep;
+}
+
 /* central control function for LCD backlight and keyboard
  * illumination interface to the outer world. Up to now only
  * module_powersave.c called this function.
@@ -741,6 +863,8 @@ display_change_brightness(int op)
 		base->flags.status = STATUS_NORMAL;
 		break;
 	}
+	display_set_lcdbrightness(base->lcdlight.target);
+	display_set_kbdbrightness(base->kbdlight.target);
 }
 
 void
