@@ -9,11 +9,12 @@
  * as published by the Free Software Foundation; either version
  * 2 of the License, or (at your option) any later version.
  * -------------------------------------------------------------------------*/
-
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
+#ifdef WITH_OSS
+
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -27,6 +28,8 @@
 
 #include "gettext_macros.h"
 #include "input_manager.h"
+#include "config_manager.h"
+#include "module_mixer.h"
 #include "module_ossmixer.h"
 #include "support.h"
 
@@ -52,12 +55,11 @@ char *mixerchannels[] = {"volume", "bass", "treble", "synth", "pcm", "speaker",
 		"phoneout", "video", "radio", "monitor"};
 
 int
-ossmixer_init (struct tagitem *taglist)
+ossmixer_init ()
 {
 	struct moddata_ossmixer *base = &modbase_ossmixer;
 	static char devbuffer_mixer[STDBUFFERLEN];
-	char *devname;
-	int rc;
+	int sid;
 
 	base->mixerdev		= devbuffer_mixer;
 	base->keyvolup		= KEY_VOLUMEUP;
@@ -73,6 +75,49 @@ ossmixer_init (struct tagitem *taglist)
 	base->flags.mute	= 0;
 	base->flags.mixer_delayed = 0;
 	base->flags.init_complete = 0;
+	base->flags.open    = 0;
+
+	devbuffer_mixer[0] = 0;
+	
+	if ((sid = registerCfgSection ("MODULE MIXER")) == 0) {
+		print_msg (PBB_ERR, _("Can't register configuration section %s, out of memory."), "MODULE MIXER");
+		return E_NOMEM;
+	} else {
+		/* options always to process */
+		registerCfgOptionString (sid, "OSS_Device", TAG_MIXERDEVICE, 0,
+				NULL);
+		registerCfgOptionString (sid, "OSS_Channels", TAG_MIXERCHANNELS, 0,
+				NULL);
+		registerCfgOptionString (sid, "dev_Mixer", TAG_MIXERDEVICE, FLG_RONLY,
+				NULL);
+		registerCfgOptionString (sid, "MixerChannels", TAG_MIXERCHANNELS, FLG_RONLY,
+				NULL);
+	
+		/* options only to process if this module is active (open) */
+		registerCfgOptionInt (sid, "Volume", TAG_VOLUME, 0,
+				N_("initial volume level"));
+		registerCfgOptionBool (sid, "Speakers_muted", TAG_MUTE, 0,
+				N_("mute after startup?"));
+		registerCfgOptionKey (sid, "VolumeUpKey", TAG_VOLUMEUPKEY, TAG_VOLUMEUPMOD, 0,
+				NULL);
+		registerCfgOptionKey (sid, "VolumeDownKey", TAG_VOLUMEDOWNKEY, TAG_VOLUMEDOWNMOD, 0,
+				NULL);
+		registerCfgOptionKey (sid, "MuteKey", TAG_MUTEKEY, TAG_MUTEMOD, 0,
+				NULL);
+		registerCfgOptionBool (sid, "MixerInitDelay", TAG_MIXERINITDELAY, 0,
+				NULL);
+	}
+	register_function (QUERYQUEUE, ossmixer_query);
+	register_function (CONFIGQUEUE, ossmixer_configure);
+	return 0;
+}
+
+int
+ossmixer_open (struct tagitem *taglist)
+{
+	struct moddata_ossmixer *base = &modbase_ossmixer;
+	char *devname;
+	int rc;
 
 	devname = (char *) tagfind (taglist, TAG_MIXERDEVICE, (long) DEFAULT_MIXER);
 	if ((rc = copy_path (devname, base->mixerdev, TYPE_CHARDEV, CPFLG_MAYBEMISSED)))
@@ -81,18 +126,17 @@ ossmixer_init (struct tagitem *taglist)
 	ossmixer_setuserdevmask ((char *) tagfind (taglist, TAG_MIXERCHANNELS, (long) "volume"));
 
 	if ((base->flags.mixer_delayed = tagfind (taglist, TAG_MIXERINITDELAY, 0)) == 0)
-		ossmixer_finish_init ();
-	else {
-		tagskip (taglist, TAG_VOLUME);  /* can't set volume with only partly initialized mixer */
+		if (ossmixer_finish_init () == -1)
+			return E_NOSUPPORT;
+
+	if (!base->flags.init_complete) {
+		/* can't set volume with only partly initialized mixer */
+		tagskip (taglist, TAG_VOLUME);
 		tagskip (taglist, TAG_MUTE);
 	}
-
-	if ((rc = ossmixer_handle_tags (MODE_CONFIG, taglist)))
-		return rc;
-
+	
+	base->flags.open = 1;
 	register_function (KBDQUEUE, ossmixer_keyboard);
-	register_function (QUERYQUEUE, ossmixer_query);
-	register_function (CONFIGQUEUE, ossmixer_configure);
 	return 0;
 }
 
@@ -108,25 +152,36 @@ ossmixer_finish_init ()
 				base->mixerdevmask = devmask;
 			else {
 				base->mixerdevmask = 1 << SOUND_MIXER_VOLUME;
-				print_error (_("WARNING: Can't get devmask from mixer [%s]; using default.\n"), base->mixerdev);
+				print_msg (PBB_WARN, _("Can't get devmask from mixer [%s]; using default.\n"), base->mixerdev);
 			}
 			base->flags.init_complete = 1;        /* mixer setup completed */
 			close (fd);
 			return 0;
 		}
-		print_error (_("ERROR: Can't get volume of master channel [%s].\n"), mixerchannels[base->master]);
+		print_msg (PBB_ERR, _("Can't get volume of master channel [%s].\n"), mixerchannels[base->master]);
 		close(fd);
 	} else if (base->flags.mixer_delayed == 0)
-		print_error(_("ERROR: Can't open mixer device [%s]. %s\n"), base->mixerdev, strerror(errno));
+		print_msg (PBB_ERR, _("Can't open mixer device [%s]. %s\n"), base->mixerdev, strerror(errno));
 	return -1;
 }
 
+int
+ossmixer_close ()
+{
+	struct moddata_ossmixer *base = &modbase_ossmixer;
+
+	base->flags.open = 0;
+	return 0;
+}
+
 int
 ossmixer_exit ()
 {
 	return 0;
 }
 
+/* ---- */
+
 void
 ossmixer_keyboard (struct tagitem *taglist)
 {
@@ -139,9 +194,9 @@ ossmixer_keyboard (struct tagitem *taglist)
 
 	if (value) {
 		if ((code == base->keyvolup) && ((mod & ~MOD_SHIFT) == base->modvolup)) {
-			step = (mod & MOD_SHIFT) ? +10 : +1;
+			step = (mod & MOD_SHIFT) ? +1 : +10;
 		} else if ((code == base->keyvoldn) && ((mod & ~MOD_SHIFT) == base->modvoldn)) {
-			step = (mod & MOD_SHIFT) ? -10 : -1;
+			step = (mod & MOD_SHIFT) ? -1 : -10;
 		} else if ((code == base->keymute) && (mod == base->modmute) && (value == 1)) {
 			step = 0;
 		} else return;
@@ -162,71 +217,89 @@ ossmixer_configure (struct tagitem *taglist)
 	ossmixer_handle_tags (MODE_CONFIG, taglist);
 }
 
-int
+void
 ossmixer_handle_tags (int cfgure, struct tagitem *taglist)
 {
 	struct moddata_ossmixer *base = &modbase_ossmixer;
-	int err, rc = 0;
+	int err;
 
 	while (taglist->tag != TAG_END) {
+
+		/* tags that would always be processed even if this module was closed
+		 */
 		switch (taglist->tag) {
-		case TAG_VOLUME:
-			if (cfgure) {
-				if ((taglist->data - base->volume) != 0)
-					ossmixer_set_and_send (taglist->data - base->volume);
-			} else
-				taglist->data = base->volume;
-			break;
-		case TAG_MUTE:
-			if (cfgure) {
-				if (base->flags.mute != taglist->data)
-					ossmixer_set_and_send (0);
-			} else
-				taglist->data = base->flags.mute;
-			break;
 		case TAG_MIXERDEVICE:
 			if (cfgure) {
 				if ((err = copy_path ((char *) taglist->data, base->mixerdev, TYPE_CHARDEV, CPFLG_MAYBEMISSED)))
-					rc = tagerror (taglist, err);
+					tagerror (taglist, err);
 			} else
 				taglist->data = (long) base->mixerdev;
 			break;
-		case TAG_VOLUMEUPKEY:
-			if (cfgure)	base->keyvolup = taglist->data;
-			else		taglist->data = base->keyvolup;
-			break;
-		case TAG_VOLUMEUPMOD:
-			if (cfgure)	base->modvolup = taglist->data;
-			else		taglist->data = base->modvolup;
-			break;
-		case TAG_VOLUMEDOWNKEY:
-			if (cfgure)	base->keyvoldn = taglist->data;
-			else		taglist->data = base->keyvoldn;
-			break;
-		case TAG_VOLUMEDOWNMOD:
-			if (cfgure)	base->modvoldn = taglist->data;
-			else		taglist->data = base->modvoldn;
-			break;
-		case TAG_MUTEKEY:
-			if (cfgure)	base->keymute = taglist->data;
-			else		taglist->data = base->keymute;
-			break;
-		case TAG_MUTEMOD:
-			if (cfgure)	base->modmute = taglist->data;
-			else		taglist->data = base->modmute;
-			break;
-		case TAG_MIXERINITDELAY:
-			if (cfgure)	base->flags.mixer_delayed  = taglist->data;
-			else		taglist->data = base->flags.mixer_delayed;
-			break;
 		case TAG_MIXERCHANNELS:
 			if (cfgure)	ossmixer_setuserdevmask ((char *) taglist->data);
-			else		taglist->data = (long) ossmixer_getuserdevmask(base->userdevmask & base->mixerdevmask);
+			else		taglist->data = (long) ossmixer_getuserdevmask(base->userdevmask);
+/*			else		taglist->data = (long) ossmixer_getuserdevmask(base->userdevmask & base->mixerdevmask);*/
 			break;
 		}
+
+		/* tags that will be processed only if the module is open.
+		 */
+		if (base->flags.open) {
+			switch (taglist->tag) {
+			case TAG_VOLUME:
+				if (cfgure) {
+					if ((taglist->data - base->volume) != 0)
+						ossmixer_set_and_send (taglist->data - base->volume);
+				} else
+					taglist->data = base->volume;
+				break;
+			case TAG_MUTE:
+				if (cfgure) {
+					if (base->flags.mute != taglist->data)
+						ossmixer_set_and_send (0);
+				} else
+					taglist->data = base->flags.mute;
+				break;
+			}
+		}
+		
+		/* tags that could be configured even if the module is closed but only be
+		 * read if the module is open.
+		 */
+		if (cfgure || base->flags.open) {
+			switch (taglist->tag) {
+			case TAG_VOLUMEUPKEY:
+				if (cfgure)	base->keyvolup = taglist->data;
+				else		taglist->data = base->keyvolup;
+				break;
+			case TAG_VOLUMEUPMOD:
+				if (cfgure)	base->modvolup = taglist->data;
+				else		taglist->data = base->modvolup;
+				break;
+			case TAG_VOLUMEDOWNKEY:
+				if (cfgure)	base->keyvoldn = taglist->data;
+				else		taglist->data = base->keyvoldn;
+				break;
+			case TAG_VOLUMEDOWNMOD:
+				if (cfgure)	base->modvoldn = taglist->data;
+				else		taglist->data = base->modvoldn;
+				break;
+			case TAG_MUTEKEY:
+				if (cfgure)	base->keymute = taglist->data;
+				else		taglist->data = base->keymute;
+				break;
+			case TAG_MUTEMOD:
+				if (cfgure)	base->modmute = taglist->data;
+				else		taglist->data = base->modmute;
+				break;
+			case TAG_MIXERINITDELAY:
+				if (cfgure)	base->flags.mixer_delayed  = taglist->data;
+				else		taglist->data = base->flags.mixer_delayed;
+				break;
+			}
+		}
 		taglist++;
 	}
-	return rc;
 }
 
 void
@@ -254,7 +327,7 @@ ossmixer_setuserdevmask (char *channels)
 char*
 ossmixer_getuserdevmask (int devmask)
 {
-        static char channels[200];
+	static char channels[200];
 	int n;
 
 	channels[0] = 0;
@@ -387,4 +460,4 @@ get_mixer_devmask(int fd)
 	return -1;
 }
 
-
+#endif /* WITH_OSS */
