@@ -4,7 +4,7 @@
  * module_peep.h
  * prototypes and definitions for the module peep.
  *
- * Copyright 2002 Matthias Grimm
+ * Copyright 2002-2005 Matthias Grimm
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,7 +20,8 @@ struct tagtable {
 };
 
 /* public prototypes */
-int peep_init (struct tagitem *taglist);
+int peep_init ();
+int peep_open (struct tagitem *taglist);
 int peep_exit ();
 
 /* private prototypes */
