@@ -20,12 +20,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <fcntl.h>
+#include <dirent.h>
 #include <errno.h>
 #include <string.h>
 #include <stdarg.h>
 #include <unistd.h>
 #include "pbbinput.h"
-#include "i2c-dev.h"
 #include <sys/ioctl.h>
 #include <linux/pmu.h>
 #include <linux/adb.h>
@@ -55,6 +55,7 @@ struct moddata_pmac {
 	int version;     /* PMU version */
 	char *identity;  /* Identity string of this laptop */
 	int machine;     /* OpenFirmware machine code */
+	int lmuaddr;	 /* i2c address of LMU Controller */
 	struct modflags_pmac flags;
 	int flagschanged;        /* PMU flags that have changed recently */
 	int oharevolbutton;      /* level of volume button on OHARE PBs scaled to 0..100 */
@@ -160,7 +161,7 @@ pmac_open (struct tagitem *taglist)
 	base->batlog_mode	= BATLOG_NONE;  /* no batlog */
 	base->batlog_cycle	= -1;         /* cycle invalid */
 	base->timeleft      = -1;
-	base->machine       = pmac_getmachine ();
+	base->machine       =  0;
 #if defined(DEBUG) && SIMUAMBIENT
 	base->ambient       = 0;
 #endif
@@ -184,34 +185,33 @@ pmac_open (struct tagitem *taglist)
 		}
 	} else return rc;
 
+	base->version = get_pmu_version(base->fd_adb);
+	base->machine = getMachineID (base->version);
+	snprintf(base->identity, STDBUFFERLEN, "%s - PMU version: %d",
+			getMachineName(base->machine), base->version);
+	
+	ioctl(base->fd_pmu, PMU_IOC_CAN_SLEEP, &val);
+	base->flags.sleepsupported = val == 1 ? 1 : 0; /* check if sleep is supported on this system */
+	
 	/* If this PowerBook has an ambient light sensor connected
-	 * to the I2C bus, search for the device and save the name
+	 * to the I2C bus, search for the device and save  name and
+	 * bus address
 	 */
-	val = -1;
-	if (haveI2CAmbient()) {
-		if ((val = findI2CBus()) >= 0)
-			snprintf(base->i2cdev, STDBUFFERLEN, "/dev/i2c-%d", val);
-		else if (val == (-E_PERM))
-			print_msg (PBB_WARN, _("Can't access i2c devices. Permission denied.\n"));
-	}
+	if ((base->lmuaddr = getLMUAddress()) != 0)
+		strncpy (base->i2cdev, findLMUDevice(base->lmuaddr), STDBUFFERLEN);
 
 	/* If we have an Ambient light sensor in this PowerBook, we
 	 * must measure the LCD feedback to this sensor in current
 	 * state. To do this we switch the backlight off for a moment
 	 * to measure the real ambient light level.
 	 */
-	if ((val >= 0) || havePMUAmbient()) {  
+	if (haveI2CAmbient() || havePMUAmbient()) {  
 		print_msg (PBB_INFO, _("LMU found, ambient light sensor and keyboard illumination active.\n"));
-		val = pmac_get_lcdbacklight();
+		val = process_queue_single (QUERYQUEUE, TAG_HW_BACKLIGHT, -1);
 		pmac_set_lcdbacklight (0);    /* measure LCD feedback */
 		pmac_set_lcdbacklight (val);  /* restore LCD backlight level */
 	}
 
-	base->version  = get_pmu_version(base->fd_adb);
-	pmac_identify(base->identity, base->version);
-	ioctl(base->fd_pmu, PMU_IOC_CAN_SLEEP, &val);
-	base->flags.sleepsupported = val == 1 ? 1 : 0; /* check if sleep is supported on this system */
-
 	pmac_update_flags ();
 	pmac_update_batteryinfo ();
 	base->flagschanged = 0;
@@ -225,20 +225,22 @@ pmac_open (struct tagitem *taglist)
     print_msg (PBB_INFO, "DBG: Ambient:  %s\n",
 			haveI2CAmbient () ? "I2C" : havePMUAmbient () ? "PMU" : "None");
 #endif
-	
-	/* Every frame buffer driver could register a backlight controller,
-	 * but only the right one is accepted. Verification is done by checking
-	 * OpenFirmware's property "backlight-control" in register_backlight_controller().
-	 * ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &val);  returns -ENODEV if no
-	 * backlight device is registered. This could be used to check if a backlight
-	 * controller is installed or if we have to call our fallback functions.
-	 */
-	
+
 	base->trackpad_mode = trackpad_get_config(); /* get trackpad mode */
 
 	if (base->version != OHARE_PMU) {
-		ioctl(base->fd_pmu, PMU_IOC_GRAB_BACKLIGHT, &DUMMY);
 		base->keyboard_mode = keyboard_get_config();	/* get keyboard config*/
+
+		/* If the kernel has been compiled with CONFIG_PMAC_BACKLIGHT_LEGACY
+		 * the following ioctl is needed to disable the ancient backlight
+		 * control of the kernel. Otherwise the new sysfs interface to the
+		 * backlight will not work correctly on PowerBooks. 
+		 * The ioctl doesn't matter if the PMU backlight driver is used or
+		 * the kernel legacy code was not compiled in.
+		 * Because this is a PowerBook speciality the ioctl is called here and
+		 * not in the sysfs backlight driver.
+		 */
+		ioctl(base->fd_pmu, PMU_IOC_GRAB_BACKLIGHT, &DUMMY); 
 	}
 
 	if ((addInputSource (base->fd_pmu, pmac_pmu_handler, NULL, FALSE)) == NULL)
@@ -249,7 +251,8 @@ pmac_open (struct tagitem *taglist)
 		register_function (KBDQUEUE, pmac_keyboard);
 	
 	register_function (MOUSEQUEUE, pmac_mouse);
-	register_function (T100QUEUE, pmac_timer);
+	register_function (T100QUEUE, pmac_timer100);
+	register_function (T1000QUEUE, pmac_timer1000);
 	register_function (SECUREQUEUE, pmac_secure);
 	return 0;
 }
@@ -412,13 +415,9 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			} else
 				taglist->data = (long) base->adbdev;
 			break;
-		case TAG_BACKLIGHTMAX:     /* private tag */
-			if (cfgure)	;
-			else		taglist->data = BACKLIGHTMAX;
-			break;
 		case TAG_BACKLIGHTLEVEL:   /* private tag */
 			if (cfgure)	pmac_set_lcdbacklight (taglist->data);
-			else		taglist->data = pmac_get_lcdbacklight();
+			else		taglist->data = process_queue_single (QUERYQUEUE, TAG_HW_BACKLIGHT, -1);
 			break;
 		case TAG_TPMODEUPKEY:
 			if (cfgure)	base->keytpmodeup = taglist->data;
@@ -507,7 +506,7 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			break;
 
 		case TAG_AMBIENTLIGHT:
-			if (!base->i2cdev[0] && !havePMUAmbient())
+			if (!haveI2CAmbient() && !havePMUAmbient())
 				tagerror (taglist, E_NOSUPPORT);
 			else if (cfgure)
 #if defined(DEBUG) && SIMUAMBIENT
@@ -526,6 +525,12 @@ pmac_handle_tags (int cfgure, struct tagitem *taglist)
 			if (cfgure)	;
 			else		taglist->data = KEYBLIGHTMAX;
 			break;
+		case TAG_SYSINFOSLAVE:
+			if (cfgure) ;
+			else {
+				taglist->data |= haveADBKeyboard () ? 0 : SYSINFO_PB_USB_KBD;
+				taglist->data |= haveADBTrackpad () ? 0 : SYSINFO_PB_USB_TPAD;
+			}
 		}
 		taglist++;
 	}
@@ -577,7 +582,50 @@ pmac_pmu_handler (int fd, gpointer user_data)
 }
 
 void
-pmac_timer (struct tagitem *taglist)
+pmac_timer100 (struct tagitem *taglist)
+{
+	struct moddata_pmac *base = &modbase_pmac;
+	struct tagitem args[6];
+	int val;
+
+	taglist_init (args);
+
+	if (base->version == OHARE_PMU) {         /* PowerBook 3400, etc */
+		if ((val = get_button(base->fd_adb, PMU_GET_VOLBUTTON)) >= 0) {
+			val = val > 64 ? 100 : val * 100 / 64;
+			if (base->oharevolbutton != val) {
+				base->oharevolbutton = val;
+				taglist_add (args, TAG_VOLUME, val);
+			}
+		}
+		if ((val = get_button(base->fd_adb, 0x49)) >= 0) {
+			val = 18 - (val >> 2);   /* This formula was derived for the usual pmac backlights */
+			if (val > 15) val = 15;  /* which uses 16 brightness steps. 0=min, 15=max */
+			if (val < 0) val = 0;
+			if (base->oharebrightbutton != val) {
+				base->oharebrightbutton = val;
+				taglist_add (args, TAG_LCDBRIGHTNESS, val);
+			}
+		}
+	}
+	
+	/* trackpad tapping control */
+	if (base->trackpad_pause) {
+		if ((--base->trackpad_pause) == 0) {
+			trackpad_set_config(base->trackpad_mode);
+#if defined(DEBUG) && TRACKPAD
+			printf ("Trackpad tapping enabled \r");
+			fflush (stdout);
+#endif	
+		}
+	}
+	
+	if (args[0].tag != TAG_END)
+		process_queue (CONFIGQUEUE, args);  /* distribute changes to other modules */
+}
+
+void
+pmac_timer1000 (struct tagitem *taglist)
 {
 	struct moddata_pmac *base = &modbase_pmac;
 	struct tagitem args[6];
@@ -611,38 +659,6 @@ pmac_timer (struct tagitem *taglist)
 		batlog_write(base->timeleft);
 	}
 
-	if (base->version == OHARE_PMU) {         /* PowerBook 3400, etc */
-		if ((val = get_button(base->fd_adb, PMU_GET_VOLBUTTON)) >= 0) {
-			val = val > 64 ? 100 : val * 100 / 64;
-			if (base->oharevolbutton != val) {
-				base->oharevolbutton = val;
-				taglist_add (args, TAG_VOLUME, val);
-			}
-		}
-		if ((val = get_button(base->fd_adb, 0x49)) >= 0) {
-			val = 18 - (val >> 2);           /* This formula was derived for the usual pmac backlights */
-			if (val > 15) val = 15;          /* which uses 16 brightness steps. 0=min, 15=max */
-			if (val < 0) val = 0;
-			if (base->oharebrightbutton != val) {
-				base->oharebrightbutton = val;
-				taglist_add (args, TAG_LCDBRIGHTNESS, val);
-			}
-		}
-	}
-	
-#if 0
-	/* trackpad tapping control */
-	if (base->trackpad_pause) {
-		if ((--base->trackpad_pause) == 0) {
-			trackpad_set_config(base->trackpad_mode);
-#if defined(DEBUG) && TRACKPAD
-			printf ("Trackpad tapping enabled \r");
-			fflush (stdout);
-#endif	
-		}
-	}
-#endif
-	
 	if (args[0].tag != TAG_END)
 		process_queue (CONFIGQUEUE, args);  /* distribute changes to other modules */
 	base->flagschanged = 0;
@@ -845,47 +861,6 @@ pmac_update_flags()
 	return (envold ^ envnew);
 }
 
-int
-pmac_getmachine ()
-{
-	char buffer[100], *token;
-	int machine = 0, n;
-	FILE *fd;
-
-	if ((fd = fopen ("/proc/cpuinfo", "r"))) {
-		while (fgets (buffer,sizeof (buffer), fd)) {
-			cleanup_buffer (buffer);
-			if ((token = strtok (buffer, ":\n"))) {
-				if (strncmp ("machine", token, 7))
-					continue;
-				
-				token = strtok (0, "\n");
-				if (!strncmp ("PowerBook", token, 9)) {
-					machine = (atoi (&token[9]) & 0xf) << 4;
-					for (n=9; token[n] != ',' && token[n] != '\n'; n++);
-					if (token[n] == ',')
-						machine |= (atoi (&token[n+1]) & 0xf);
-				}
-				break;
-			}
-		}
-		fclose(fd);
-	}
-	return machine;
-}
-
-int
-pmac_get_lcdbacklight ()
-{
-	struct moddata_pmac *base = &modbase_pmac;
-	long val = 0;
-		
-	if ((ioctl(base->fd_pmu, PMU_IOC_GET_BACKLIGHT, &val)) == -1)
-		return -1;
-
-	return (int) val;
-}
-
 void
 pmac_set_lcdbacklight (int val)
 {
@@ -896,7 +871,7 @@ pmac_set_lcdbacklight (int val)
 		return;
 	
 	ambient = getRawAmbient () - base->lcdfeedback;
-	ioctl(base->fd_pmu, PMU_IOC_SET_BACKLIGHT, &val);
+	process_queue_single (CONFIGQUEUE, TAG_HW_BACKLIGHT, val);
 	base->lcdfeedback = val ? getRawAmbient () - ambient : 0;
 
 #if defined(DEBUG) && (TESTAMBIENT || SIMUAMBIENT)
@@ -910,30 +885,6 @@ pmac_set_lcdbacklight (int val)
 }
 
 /* ----------------------------  GENERIC FUNCTIONS ----------------------------- */
-
-void
-pmac_identify (char* buffer, int pmu)
-{
-	int idx;
-	char *version[] = {
-		N_("Unknown PowerBook"),
-		"PowerBook 2400/3400/3500",        /*   9  */
-		"G3 PowerBook Wallstreet",          /*  10  */
-		"1999 G3 PowerBook Lombard",         /*  11  */
-		"iBook/G3 PB Pismo/G4 PB Titanium",   /*  12  */
-		"Alluminum PowerBook",                 /*  17  */
-		N_("Wow, later than an Alluminum PB! ;-)")
-	};
-  
-	if (pmu < 8)         idx = 0;
-	else if (pmu <= 12)  idx = pmu - 8;
-	else if (pmu < 17)   idx = 0;
-	else if (pmu == 17)  idx = 5;
-	else                 idx = 6;
-	
-	snprintf(buffer, STDBUFFERLEN, "%s (PMU version: %d)", version[idx], pmu);
-}
-
 int
 isModifier (int code, int mod)
 {
@@ -965,13 +916,13 @@ haveADBKeyboard ()
 	if (base->machine > 0) {
 		if (base->machine < 0x56)              /*  0 - 56 -> true  */
 			return 1;
-		if (base->machine > 0x68)              /* 69 - ?? -> false */
+		if (base->machine > 0x67)              /* 68 - ?? -> false */
 			return 0;
 		if (((base->machine >> 4) & 0xf) == 5)
 			if (base->machine >= 0x56)         /* 56 - 5f -> false */
 				return 0;
 		if (((base->machine >> 4) & 0xf) == 6)
-			if (base->machine <= 0x68)         /* 60 - 68 -> true  */
+			if (base->machine <= 0x67)         /* 60 - 67 -> true  */
 				return 1;
 	}
 	return 0;
@@ -983,7 +934,7 @@ haveADBTrackpad ()
 	struct moddata_pmac *base = &modbase_pmac;
 	
 	if (base->machine > 0) {
-		if (base->machine < 0x56)              /*  0 - 56 -> true  */
+		if (base->machine < 0x56)              /*  0 - 55 -> true  */
 			return 1;
 		if (base->machine > 0x67)              /* 68 - ?? -> false */
 			return 0;
@@ -1006,7 +957,8 @@ haveI2CAmbient ()
 #endif
 	
 	if ((base->machine >= 0x51) && (base->machine <= 0x57))
-		return 1;
+		if (base->lmuaddr != 0 && strlen(base->i2cdev) > 0)   
+			return 1;
 	return 0;
 }
 
@@ -1038,7 +990,7 @@ keyboard_get_config ()
 	} else if ((fd = open (PATH_FNMODE, O_RDONLY)) >= 0) {
 		if ((n = read (fd, ADBBuffer, ADB_BUFSIZE-1)) > 0) {
 			ADBBuffer[n] = 0; /* terminate read buffer */
-			config = atoi(ADBBuffer);
+			config = atoi((const char*)ADBBuffer);
 #ifdef DEBUG
 			printf ("DBG: Keyboard config read: '%s' -> %d.\n", ADBBuffer, config);			
 #endif
@@ -1065,8 +1017,8 @@ keyboard_set_config (int config)
 			}
 		}
 	} else if ((fd = open (PATH_FNMODE, O_WRONLY)) >= 0) {
-		sprintf (ADBBuffer, "%d", config);
-		write (fd, ADBBuffer, strlen(ADBBuffer));
+		sprintf ((char*)ADBBuffer, "%d", config);
+		write (fd, ADBBuffer, strlen((const char*)ADBBuffer));
 #ifdef DEBUG
 		printf ("DBG: Keyboard config written: '%s' -> %d.\n", ADBBuffer, config);			
 #endif
@@ -1179,6 +1131,64 @@ trackpad_set_prgmode (int fd, int dev, int mode)
 
 /* ----------------------------- LOW LEVEL ADB FUNCTIONS ---------------------------- */
 
+#if defined(DEBUG) && PMUTALK
+/* Debug functions to track down mysterious shutdowns and freezes */
+#define DEBUGLOG "/var/log/pbbuttonsd"
+
+void
+debug_wrtcmd (char *title, unsigned char *buffer, int size)
+{
+	struct timeval tv;
+	FILE *fd;
+	int i;
+
+	gettimeofday(&tv, NULL);
+	if ((fd = fopen(DEBUGLOG, "a")) != NULL) {    /* open protocol file */
+		fprintf (fd, "%ld.%4d %-20s (%d)", tv.tv_sec, (int)tv.tv_usec/1000, title, size);
+		for (i=0; i < size; i++) {
+			fprintf (fd, " %02x", buffer[i]);
+		}
+		fclose(fd);
+	}
+}
+
+void
+debug_wrtrc (int rc, int expected)
+{
+	FILE *fd;
+
+	if ((fd = fopen(DEBUGLOG, "a")) != NULL) {    /* open protocol file */
+		if (rc == -1 || rc != expected)
+			fprintf(fd, "\t-- FAIL\n");
+		else
+			fprintf(fd, "\t-- ok\n");
+		fclose(fd);
+	}
+}
+
+void
+debug_rdcmd (char *title, unsigned char *buffer, int size)
+{
+	struct timeval tv;
+	FILE *fd;
+	int i;
+
+	gettimeofday(&tv, NULL);
+	if ((fd = fopen(DEBUGLOG, "a")) != NULL) {    /* open protocol file */
+		fprintf (fd, "%ld.%4d %-20s (%d)", tv.tv_sec, (int)tv.tv_usec/1000, title, size);
+		if (size >= 0) {
+			for (i=0; i < size; i++) {
+				fprintf (fd, " %02x", buffer[i]);
+			}
+			fprintf(fd, "\t-- ok\n");
+		} else
+			fprintf(fd, "\t-- FAIL\n");
+		fclose(fd);
+	}
+}
+/* ---------------------------------------------------------------*/
+#endif
+
 /* This function sends a request to the PMU via the ADB bus and reads
    the result. If successful the count of valid data bytes in the buffer will be
    returned, otherwise -1 */
@@ -1198,11 +1208,21 @@ send_pmu_request(int fd, unsigned char *buffer, int params, ...)
 		buffer[x+1] = va_arg(list, int);
 	va_end(list);
 
+#if defined(DEBUG) && PMUTALK
+	debug_wrtcmd("PMU request", buffer, x+1);
+#endif
 	n = write(fd, buffer, x+1);
+#if defined(DEBUG) && PMUTALK
+	debug_wrtrc(n, x+1);
+#endif
 	if ((n != x+1) || (n == -1))
 		return -1;
-	if ((n = read(fd, buffer, ADB_BUFSIZE)) < 0)
-		return -1;
+
+	n = read(fd, buffer, ADB_BUFSIZE);
+#if defined(DEBUG) && PMUTALK
+	debug_rdcmd("PMU answer", buffer, n);
+#endif
+	if (n < 0) return -1;
 	return n;
 }
 
@@ -1217,9 +1237,20 @@ adb_read_reg(int fd, unsigned char *ADBBuffer, int dev, int reg)
 
 	ADBBuffer[0] = ADB_PACKET;
 	ADBBuffer[1] = ADB_READREG(dev, reg);
-	if ((write(fd, ADBBuffer, 2)) != 2)
-		return -1;
+	
+#if defined(DEBUG) && PMUTALK
+	debug_wrtcmd("ADB read register", ADBBuffer, 2);
+#endif
+	n = write(fd, ADBBuffer, 2);
+#if defined(DEBUG) && PMUTALK
+	debug_wrtrc(n, 2);
+#endif
+	if (n != 2) return -1;
+	
 	n = read(fd, ADBBuffer+1, ADB_BUFSIZE-1);
+#if defined(DEBUG) && PMUTALK
+	debug_rdcmd("ADB read answer", ADBBuffer, n);
+#endif
 	return n;
 }
 
@@ -1233,9 +1264,20 @@ adb_write_reg(int fd, unsigned char *ADBBuffer, int len, int dev, int reg)
 	len++;     /* take care about the command byte */
 	ADBBuffer[0] = ADB_PACKET;
 	ADBBuffer[1] = ADB_WRITEREG(dev, reg);
-	if ((write(fd, ADBBuffer, len)) != len)
-		return -1;
+	
+#if defined(DEBUG) && PMUTALK
+	debug_wrtcmd("ADB write register", ADBBuffer, 2);
+#endif
+	n = write(fd, ADBBuffer, len);
+#if defined(DEBUG) && PMUTALK
+	debug_wrtrc(n, len);
+#endif
+	if (n != len) return -1;
+	
 	n = read(fd, ADBBuffer+1, ADB_BUFSIZE-1);
+#if defined(DEBUG) && PMUTALK
+	debug_rdcmd("ADB write answer", ADBBuffer, n);
+#endif
 	return 0;
 }
 
@@ -1282,44 +1324,115 @@ get_int_environment(int fd)
 
 /* ----------------------------- LOW LEVEL LMU FUNCTIONS ---------------------------- */
 
-/* This function tries to find the I2C device that controls the keyboard
- * illumination and the ambient light sensor used in some alluminum PowerBooks
- * It returns the device number on success, otherwise a negative error code.
- * The loop starts with device 4 because that's the correct device on
- * 17" alluminum PowerBooks. Device 0 could be errornous detected because
- * the function uses only circumstantial evidence to detect the correct
- * device.
+int
+addPath(char *path, int maxlen, char *pattern)
+{
+	DIR *dh;
+	struct dirent *dir;
+	int rc = 1;
+	
+	if ((dh = opendir(path))) {
+		while ((dir = readdir(dh))) {
+			if ((strncmp(dir->d_name, pattern, strlen(pattern)) == 0)) {
+				strncat(path, "/", maxlen-1);
+				strncat(path, dir->d_name, maxlen-1);
+				rc = 0;
+				break;
+			}			
+		}
+		closedir(dh);
+	}
+	return rc;
+}
+
+/* This function searches the device-tree for the lmu-controller to
+ * read its I2C bus address. Two device paths are searched:
+ *   uni-n/i2c/lmu-controller/reg or
+ *   uni-n/i2c/i2c-bus/lmu-micro/reg
+ *
+ * If the file 'reg' is found it contents divided by two will be
+ * returned. Otherwise the result is 0.
  */
 int
-findI2CBus ()
+getLMUAddress()
 {
-#if defined(DEBUG) && SIMUAMBIENT
-	int rc = 99; /* dummy file handle for i2c bus, will hopefully never be used */
-#else
-	char i2cdevice[20], buf[4];
-	int n, fd, rc = -E_NOCHAR;
+	char path[200];
+	long reg;
+	int fd, n, rc = 0, err = 0;
 
-	for(n=4; n < 260; n++) {
-		snprintf(i2cdevice, 19, "/dev/i2c-%d", (n & 255) );
-		if ((fd = open (i2cdevice, O_RDWR)) >= 0 ) {
-			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 ) {
-				if (read (fd, buf, 4) == 4 ) {
-					close (fd);
-					rc = (n & 255);
-					break;
+	path[0] = 0; /* terminate path buffer */
+	strncat(path, OFBASE, sizeof(path)-1);
+	if ((err = addPath(path, sizeof(path), "uni-n")) == 0)
+		if ((err = addPath(path, sizeof(path), "i2c")) == 0)
+			if ((err = addPath(path, sizeof(path), "lmu-controller")) != 0)
+				if ((err = addPath(path, sizeof(path), "i2c-bus")) == 0)
+					err = addPath(path, sizeof(path), "lmu-micro");
+	strncat(path, "/reg", sizeof(path)-1);
+
+	if (err == 0 && (fd = open(path, O_RDONLY)) >= 0) {
+		n = read(fd, &reg, sizeof(long));
+		if (n == sizeof(long))
+			rc = (int) (reg >> 1);
+		close(fd);
+	}
+	return rc;
+}
+
+int
+probeLMU(char *device, int addr)
+{
+	char buffer[4];
+	int fd, rc = 0;
+
+	if ((fd = open(device, O_RDWR)) >= 0) {
+		if (ioctl(fd, I2C_SLAVE, addr) >= 0) {
+			if (read (fd, buffer, 4) == 4) 
+				rc = 1;
+		}
+		close(fd);
+	} else
+		print_msg (PBB_ERR, _("Can't access i2c device %s - %s.\n"), device, strerror(errno));
+	return rc;
+}
+
+/* This function tries to find the I2C device that controls the keyboard
+ * illumination and the ambient light sensor used in some alluminum PowerBooks
+ * It returns the device file name on success or an empty string, if no LMU
+ * controller could be found.
+ * The function uses the sysfs to find the appropriate i2c device to which the
+ * LMU controller is connected to. The kernel module i2c-dev must be loaded
+ * for this function to work.
+ */
+const char*
+findLMUDevice(int addr)
+{
+	static char buffer[40];
+	DIR *dh;
+	struct dirent *dir;
+	unsigned int n;
+	int fd;
+
+	if ((dh = opendir(SYSI2CDEV))) {
+		while ((dir = readdir(dh))) {
+			if (dir->d_name[0] == '.') continue;
+			snprintf(buffer, sizeof(buffer), SYSI2CDEV"/%s/name", dir->d_name);
+			if ((fd = open(buffer, O_RDONLY)) >= 0) {
+				n = read(fd, buffer, sizeof(buffer));
+				close(fd);
+				if (n > 0 && n < sizeof(buffer)) {
+					buffer[n-1] = 0;
+					if ((strncmp(I2CCHIP" ", buffer, 6) == 0)) {
+						snprintf(buffer, sizeof(buffer), "/dev/%s", dir->d_name);
+						if ((probeLMU(buffer, addr)))
+							break;
+						buffer[0] = 0;
+					}
 				}
 			}
-			close (fd);
-		} else if (errno == ENODEV) {
-			rc = -E_NOCHAR;
-			break;
-		} else if (errno == EACCES) {
-			rc = -E_PERM;
-			break;
 		}
+		closedir(dh);
 	}
-#endif
-	return rc;
+	return buffer;
 }
 
 /* This function returns the a compensated ambient light sensor
@@ -1355,7 +1468,7 @@ getRawAmbient ()
 	int fd, rc = -1, rc2;
 	
 #if defined(DEBUG) && SIMUAMBIENT
-	int ambient, level, idx;
+	int ambient, level, idx, max;
 
 	/* this array contains sensor values depending on the
 	 * LCD brightness without additional ambient light.
@@ -1369,15 +1482,16 @@ getRawAmbient ()
 	int asens[] = {2, 182, 217, 248, 309, 337, 381, 423, 461,
 		           499, 538, 576, 620, 1072, 1073, 1077};
 		       
-	level = pmac_get_lcdbacklight();
-	idx = level * sizeof(asens) / sizeof(int) / (BACKLIGHTMAX + 1);
+	level = process_queue_single (QUERYQUEUE, TAG_HW_BACKLIGHT, -1);
+	max   = process_queue_single (QUERYQUEUE, TAG_BACKLIGHTMAX, -1);
+	idx = level * sizeof(asens) / sizeof(int) / (max + 1);
 	ambient = asens[level == -1 ? 0 : idx] + base->ambient;
 	if (ambient > 1600) ambient = 1600;
 	return ambient;
 #else	
-	if ((haveI2CAmbient()) && (base->i2cdev[0])) {
+	if (haveI2CAmbient()) {
 		if ((fd = open (base->i2cdev, O_RDWR)) >= 0 ) {
-			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 )
+			if (ioctl (fd, I2C_SLAVE, base->lmuaddr) >= 0 )
 				if (read (fd, buf, 4 ) == 4) {
 					rc  = (buf[0] << 8) | buf[1];
 					rc2 = (buf[2] << 8) | buf[3];
@@ -1408,24 +1522,29 @@ setKBDIllumination (unsigned short level)
 
 	if (level > KEYBLIGHTMAX) level = KEYBLIGHTMAX; /* check bounds */
 	
-	if ((haveI2CAmbient ()) && base->i2cdev[0]) {
+	if (haveI2CAmbient()) {
 		buf[0] = 0x01;   /* i2c register */
-		buf[1] = (unsigned char) (( ( level * 255 ) & 0xFF00 ) >> 8 );
-		buf[2] = (unsigned char) (( ( level * 255 ) & 0x00FF ));
+
+		/* The format appears to be:
+		 *
+		 *          byte 1   byte 2
+		 *         |<---->| |<---->|
+		 *         xxxx7654 3210xxxx
+		 *             |<----->|
+		 *                 ^-- brightness
+		 */
+		
+		buf[1] = (unsigned char) level >> 4;
+		buf[2] = (unsigned char) (level & 0x0F) << 4;
 
 		if ((fd = open (base->i2cdev, O_RDWR)) >= 0 ) {
-			if (ioctl (fd, I2C_SLAVE, LMU_ADDR) >= 0 )
+			if (ioctl (fd, I2C_SLAVE, base->lmuaddr) >= 0 )
 				write (fd, buf, 3);
 			close (fd);
 		}
 	}
 
 	if (havePMUAmbient ()) {
-		/* The keyboard illumination connected to the PMU has 256 brightness
-		 * steps and because pbbuttonsd handles only 16 steps in general, we
-		 * use only the high nibble here.
-		 */
-		level = level * 255 / KEYBLIGHTMAX;
 		send_pmu_request (base->fd_adb, buf, 4, 0x4F, 0, 0, level);
 	}
 #else
@@ -1435,6 +1554,91 @@ setKBDIllumination (unsigned short level)
 #endif
 }
 
+/* -------------------- LOW LEVEL OPEN FIRMWARE FUNCTIONS -------------------- */
+int
+getMachineID(int pmu)
+{
+	char buffer[32];
+	int fd, n, machine = 0;
+
+	if ((fd = open(OFBASE"/model", O_RDONLY))) {
+		if ((n = read(fd, buffer, sizeof(buffer) - 1)) != -1) {
+			buffer[n] = 0;   /* terminate buffer, only to be sure */
+			if (strncmp("PowerBook", buffer, 9) == 0) {
+				if (buffer[9] == 0)
+					/* Dummy codes for pre-Lombard PowerBooks */
+					machine = pmu - OHARE_PMU + 1;
+				else {
+					machine = (atoi(&buffer[9]) & 0xf) << 4;
+					for (n = 9; buffer[n] != ',' && buffer[n] != '\0'; ++n);
+					if (buffer[n] == ',')
+						machine |= atoi(&buffer[n+1]) & 0xf;
+				}
+			}
+		}
+		close(fd);
+	}
+	return machine;
+}
+
+const char*
+getMachineName(int mid)
+{
+	unsigned int x, y;
+	static char *machines[7][10] = {
+	{ N_("Unknown PowerBook"),
+	  "PowerBook 3400",                        /* 0,1 */
+	  "PowerBook 3500",                        /* 0,2 */
+	  "PowerBook Wallstreet (Apr 1998)",       /* 0,3 */
+	  NULL, NULL, NULL, NULL, NULL, NULL },
+	{ NULL,
+	  "PowerBook 101 Lombard (Apr 1999)",      /* 1,1 */
+	  NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL },
+	{ NULL,
+	  "iBook (Feb 2000)",                      /* 2,1 */
+	  "iBook FireWire (Sep 2000)",             /* 2,2 */
+	  NULL, NULL, NULL, NULL, NULL, NULL, NULL },
+	{ NULL,
+	  "PowerBook G3 Pismo (Feb 2000)",         /* 3,1 */
+	  "PowerBook G4 Titanium (Dec 2000)",      /* 3,2 */
+	  "PowerBook G4 Titanium II (Oct 2001)",   /* 3,3 */
+	  "PowerBook G4 Titanium III (Apr 2002)",  /* 3,4 */
+	  "PowerBook G4 Titanium IV (Nov 2002)",   /* 3,5 */
+	  NULL, NULL, NULL, NULL },
+	{ NULL,
+	  "iBook 2 (May 2001)",                    /* 4,1 */
+	  "iBook 2 (May 2002)",                   /* 4,2 */
+	  "iBook 2 rev. 2 (Nov 2002)",           /* 4,3 */
+	  NULL, NULL, NULL, NULL, NULL, NULL },
+	{ NULL,
+	  "PowerBook G4 17\" (Mar 2003)",      /* 5,1 */
+	  "PowerBook G4 15\" (Sep 2003)",      /* 5,2 */
+	  "PowerBook G4 17\" (Sep 2003)",      /* 5,3 */
+	  "PowerBook G4 15\" (Apr 2004)",      /* 5,4 */
+	  "PowerBook G4 17\" (Apr 2004)",      /* 5,5 */
+	  "PowerBook G4 15\" (Feb 2005)",      /* 5,6 */
+	  "PowerBook G4 17\" (Feb 2005)",      /* 5,7 */
+	  "PowerBook G4 15\" (Oct 2005)",      /* 5,8 */
+	  "PowerBook G4 17\" (Oct 2005)" },    /* 5,9 */
+	{ NULL,
+	  "PowerBook G4 12\" (Jan 2003)",      /* 6,1 */
+	  "PowerBook G4 12\" (Sep 2003)",      /* 6,2 */
+	  "iBook G4 (Oct 2003)",               /* 6,3 */
+	  "PowerBook G4 12\" (Apr 2004)",      /* 6,4 */
+	  "iBook G4 (Apr 2004)",               /* 6,5 */
+	  NULL,
+	  "iBook G4 (Jul 2005)",               /* 6,7 */
+	  "PowerBook G4 12\" (Oct 2005)",      /* 6,8 */
+	  NULL }
+	};
+	
+	x = mid >> 4 & 0x0f;
+	y = mid & 0xf;
+	if (x < 7 && y < 10 && machines[x][y] != NULL)
+		return machines[x][y];
+	else
+		return machines[0][0];
+}
 
 /* --------------------- BATTERY LOGFILE DEBUG FUNCTIONS --------------------- */
 int
