@@ -15,13 +15,17 @@
 #include <glib.h>
 #include <pbbtaglist.h>
 
+#ifndef I2C_SLAVE
+#define I2C_SLAVE   0x0703	/* Change slave address (7 or 10 bits) */
+#endif
+#define I2CCHIP		"uni-n"
+
 #define ADB_BUFSIZE		32      /* Size of the ADB buffer, constant,
                                             must not be changed !! */
 
 #define OHARE_PMU		9
 
-#define BACKLIGHTMAX		15      /* from asm/backlight.h */
-#define KEYBLIGHTMAX		15
+#define KEYBLIGHTMAX		255
 #define MAX_BATTERIES		2
 
 #define DEFAULT_PMU		"/dev/pmu"
@@ -29,6 +33,8 @@
 #define DEFAULT_BATLOG		"/var/lib/pbbuttons/%03d_battery.log"
 #define DEFAULT_BATCYCLE	"/var/lib/pbbuttons/battery.cycle"
 #define PATH_FNMODE		"/sys/module/usbhid/parameters/pb_fnmode"
+#define OFBASE			"/proc/device-tree"
+#define SYSI2CDEV		"/sys/class/i2c-dev"
 
 /* definitions for input device config */
 #define TRACKPAD_NOTAP_NAME	"notap"
@@ -58,8 +64,6 @@ enum {BATLOG_NONE, BATLOG_CYCLE, BATLOG_LOG, BATLOG_LAST=BATLOG_LOG};
 #define PMU_ENV_BATTERY_PRESENT	0x10
 #endif
 
-#define LMU_ADDR	0x42
-
 struct modflags_pmac {
 	unsigned int coveropen :1;
 	unsigned int ac_power:1;
@@ -88,16 +92,16 @@ void pmac_query (struct tagitem *taglist);
 void pmac_configure (struct tagitem *taglist);
 void pmac_handle_tags (int cfgure, struct tagitem *taglist);
 gboolean pmac_pmu_handler (int fd, gpointer user_data);
-void pmac_timer (struct tagitem *taglist);
+void pmac_timer100 (struct tagitem *taglist);
+void pmac_timer1000 (struct tagitem *taglist);
 
 void pmac_update_batteryinfo ();
 int pmac_get_procac ();
 int pmac_update_flags ();
-int pmac_getmachine ();
 void activate_sleepmode ();
-int pmac_get_lcdbacklight();
 void pmac_set_lcdbacklight(int val);
-void pmac_identify (char* buffer, int pmu);
+int getMachineID(int pmu);
+const char* getMachineName(int mid);
 int isModifier (int code, int mod);
 int haveADBKeyboard ();
 int haveADBTrackpad ();
@@ -125,7 +129,10 @@ void batlog_save();
 void batlog_write(long timerem);
 
 /* functions for ambient light sensor and keyboard illumination */
-int findI2CBus ();
+int addPath(char *path, int maxlen, char *pattern);
+int getLMUAddress();
+int probeLMU(char *device, int addr);
+const char *findLMUDevice(int addr);
 int getAmbient ();
 int getRawAmbient ();
 void setKBDIllumination (unsigned short level);
