@@ -55,8 +55,8 @@ struct moddata_power {
 	int powersource;    /* REDUNDANT copy from module pmac */
 	int timeremaining;   /* REDUNDANT copy from module pmac
 							or filled by IBaM */
-	int mode;             /* module mode: awake, sleep, dim. */
-	int sleeptime;         /* counter */
+	int mode;              /* module mode: awake, sleep, dim. */
+	unsigned int sleeptime; /* counter */
 	int sleepcountdown;     /* another counter */
 	int warnlevel;           /* current battery warnlevel */
 	struct timeval tv;       /* time of key press */
@@ -93,8 +93,9 @@ power_init ()
 	static char scriptbuffer2[STDBUFFERLEN];
 #ifdef WITH_IBAM
 	static char scriptbuffer3[STDBUFFERLEN];
+	int err;
 #endif
-	int sid, err;
+	int sid;
 
 	bzero (base, sizeof (struct moddata_power));
 	bzero (scriptbuffer1, STDBUFFERLEN);  /* no default for script_pmcs */
@@ -725,7 +726,10 @@ power_timer1000 (struct tagitem *taglist)
 {
 	struct moddata_power *base = &modbase_power;
 	static int cputimer = 0, nettimer = 0, beeptimer = 0;
-	int cpuload = 0, netload = 0, time;
+	int cpuload = 0, netload = 0;
+#ifdef WITH_IBAM
+	int time;
+#endif
 
 	if (base->flags.cpuload_enable) {
 		cpuload = power_read_cpu ();
@@ -1018,16 +1022,17 @@ send_sigpwr (char mode)
   return err;
 }
 
-/* This function reads /proc/stat and calculates the current cpuload.
-    If an error occured, the return value would be '0', so that no program
-    features espacially sleep will be blocked. */
-
+/* This function reads /proc/stat and calculates the current
+ * cpuload. If an error occured, the return value would be
+ * '0', so that no program features especially sleep will
+ * be blocked.
+ */
 int
 power_read_cpu (void)
 {
+	static ulong pvalue = 0, ptotal = 0;
 	char buffer[1024], *token;
-	static ulong pvalue = 0, ptotal;
-	ulong value = 0, total = 1, user, nice, sys, idle;
+	ulong value, total, user, nice, sys, idle, usn;
 	FILE *fd;
 
 	if ((fd = fopen ("/proc/stat","r"))) {
@@ -1042,13 +1047,13 @@ power_read_cpu (void)
 			}
 		fclose(fd);
 
-		if (pvalue) {
-			value = user + sys + nice - pvalue;
-			total = user + sys + nice + idle - ptotal;
-		}
-		pvalue = user + sys + nice;
-		ptotal = pvalue + idle;
-		return (int) (value * 100 / total);
+		usn    = user + sys + nice;
+		value  = usn - pvalue;
+		pvalue = usn;
+		total  = usn + idle - ptotal;
+		ptotal = usn + idle;
+		if (total)
+			return (int) (value * 100 / total);
 	}
 	return 0;
 }
