@@ -4,7 +4,7 @@
  * module_system.h
  * prototypes and definitions for general system functions.
  *
- * Copyright 2002 Matthias Grimm
+ * Copyright 2002-2005 Matthias Grimm
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -17,13 +17,15 @@
 #define RESCANTIME	5	  /* in seconds */
 
 /* public prototypes */
-int system_init (struct tagitem *taglist);
+int system_init ();
+int system_open (struct tagitem *taglist);
+int system_close ();
 int system_exit ();
 
 /* private prototypes */
 void system_configure (struct tagitem *taglist);
 void system_query (struct tagitem *taglist);
-int system_handle_tags (int cfgure, struct tagitem *taglist);
+void system_handle_tags (int cfgure, struct tagitem *taglist);
 void system_timer1000 (struct tagitem *taglist);
 void system_config_writable (char *configreadfile, char *configwritefile);
 
