@@ -82,7 +82,7 @@ copy_path (char *source, char *dest, int type, int flags)
 			break;
 		default:
 			if (whitespace != NULL) *whitespace = ' ';
-			strncpy(dest, source, STDBUFFERLEN);
+			if (dest) strncpy(dest, source, STDBUFFERLEN);
 			break;
 		}
 	} else {
