@@ -16,6 +16,15 @@
 #define CPFLG_MAYBEMISSED    1
 #define CPFLG_PBBONLY		 2
 
+struct ChildData {
+	char **argv;
+	GPid pid;
+	gint twatch;
+	gint cwatch;
+	gint ref;     /* reference count */
+	gint rc;      /* child's return code, or -1 */
+};
+
 /* prototypes */
 int copy_path (char *source, char *dest, int type, int flags);
 int call_script (char* template, ...);
